@@ -18,7 +18,7 @@ again.
 =============================================================================="
 	fi    
 	echo
-	if [ "$DELALL" = "1" ] && [ "$NAMEPKG" != "" ]; then
+	if [ "$DELALL" = "on" ] && [ "$NAMEPKG" != "" ]; then
 		rm $TEMP/$NAMEPKG &>/dev/null
 	fi		
 	( rm -f /var/lock/slackpkg.$$ && rm -rf $TMPDIR ) &>/dev/null
@@ -29,6 +29,17 @@ trap 'cleanup' 2 14 15 		# trap CTRL+C and kill
 # Syntax Checking
 #
 function system_checkup() {
+	# Check if the config files are updated to the new slackpkg version
+	#
+	if [ "$WORKDIR" = "" ]; then
+		echo -e "\
+\nYou need to upgrade your slackpkg.conf.\n\
+This is a new slackpkg version and many changes happens in config files.\n\
+In ${CONF}/slackpkg.conf.new there is a sample of the new configuration.\n\
+\nAfter update your configuration run: slackpkg update\n" 
+		cleanup
+	fi
+
 	# Checking if another instance of slackpkg is running
 	#
 	if [ "`ls /var/lock/slackpkg.* 2>/dev/null`" ] && \
@@ -54,7 +65,7 @@ Fix it and run slackpkg again.\n"
 
 	# Checking if is the first time running slackpkg
 	#                                               
-	if ! [ -f ${CONF}/pkglist ] && [ "$CMD" != "update" ]; then
+	if ! [ -f ${WORKDIR}/pkglist ] && [ "$CMD" != "update" ]; then
 		echo -e "\
 \nHey! Probably this is the first time you are running slackpkg.\n\
 Before install|upgrade|reinstall anything, you need to uncomment\n\
@@ -128,10 +139,10 @@ Please, log as root or call your system administrator.\n"
 	# md5sum checks
 	#
 	if ! [ `which md5sum 2>/dev/null` ]; then
-		CHECKPKG=0
-	elif ! [ -f ${CONF}/CHECKSUMS.md5 ] && \
+		CHECKPKG=off
+	elif ! [ -f ${WORKDIR}/CHECKSUMS.md5 ] && \
 		[ "$CMD" != "update" ] && \
-		[ "$CHECKPKG" = "1" ]; then
+		[ "$CHECKPKG" = "on" ]; then
 		echo -e "\n\
 No CHECKSUMS.md5 found! Please, disable md5sums checking\n\
 on your ${CONF}/slackpkg.conf or run slackpkg update\n\
@@ -150,12 +161,12 @@ Without awk, slackpkg doesn't work in any way.\n"
 
 	# Check if gpg is enabled but no GPG command are found.
 	#
-	if ! [ "$(which gpg 2>/dev/null)" ] && [ "${NOGPG}" = "0" ]; then
-		NOGPG=1
+	if ! [ "$(which gpg 2>/dev/null)" ] && [ "${CHECKGPG}" = "on" ]; then
+		CHECKGPG=off
 		echo -e "\n\
 No gpg found!!! Please, disable GPG in ${CONF}/slackpkg.conf or install\n\
 the gnupg package.\n\n\
-To disable GPG, edit slackpkg.conf and add one line with NOGPG=1.\n\
+To disable GPG, edit slackpkg.conf and add one line with CHECKGPG=off.\n\
 You can see an example in slackpkg.conf.new.\n "
 		sleep 5
 	fi 
@@ -165,13 +176,13 @@ You can see an example in slackpkg.conf.new.\n "
 	GPGFIRSTTIME="`gpg --list-keys \"$SLACKKEY\" 2>/dev/null \
 			| grep -c \"$SLACKKEY\"`"
 	if [ "$GPGFIRSTTIME" = "0" ] && [ "$CMD" != "search" ] && [ "$CMD" != "info" ] && \
-			[ "$CMD" != "update" ] && [ "$NOGPG" != "1" ]; then
+			[ "$CMD" != "update" ] && [ "$CHECKGPG" = "on" ]; then
 		echo -e "\n\
 You need the GPG key of $SLACKKEY.\n\
 To download and install that key, run:\n\n\
 \t# slackpkg update gpg\n\n\
 You can disable GPG checking, too. But it isn't a good idea.\n\
-To disable GPG, edit slackpkg.conf and add one line with NOGPG=1.\n\
+To disable GPG, edit slackpkg.conf and add one line with CHECKGPG=off.\n\
 You can see an example in slackpkg.conf.new.\n"
 		cleanup
 	fi
@@ -233,7 +244,10 @@ instead x11-devel, x11-docs, etc), or even slackware series
 # Verify if the package was corrupted by checking md5sum
 #
 function checkpkg() {
-	MD5ORIGINAL=`grep "${NAMEPKG}$" ${CONF}/CHECKSUMS.md5| cut -f1 -d \ `
+	local MD5ORIGINAL
+	local MD5DOWNLOAD
+
+	MD5ORIGINAL=`grep "${NAMEPKG}$" ${WORKDIR}/CHECKSUMS.md5| cut -f1 -d \ `
 	MD5DOWNLOAD=`md5sum ${TEMP}/${1} | cut -f1 -d \ `
 	if [ "$MD5ORIGINAL" = "$MD5DOWNLOAD" ]; then
 		echo 1 
@@ -243,155 +257,177 @@ function checkpkg() {
 }
 
 function checkgpg() {
-	gpg --verify $TEMP/${1}.asc $TEMP/$1 2>/dev/null && echo "1" || echo "0"
+	gpg --verify ${TEMP}/${1}.asc ${TEMP}/${1} 2>/dev/null && echo "1" || echo "0"
 }
 
-function check_blacklist_priority() {
-	# That rule will check if the package are in 
-	# blacklist. And later select the high priority pack
-	TPKGNAME=`echo $PKGTEST | awk -F/ '{ print $NF }'`
-	TDIRNAME=`dirname $PKGTEST | cut -d. -f2- `
-	TBASENAME=`cutpkg $TPKGNAME`
-	if grep -qx "\(${TBASENAME}\|${TDIRNAME}\)" ${CONF}/blacklist ; then 
-		PKGFOUND=0
+
+function checkblacklist {
+        local i
+        local BLKNAME="${PKGDATA[1]}"
+
+        for i in 2 3 4; do
+                if grep -qx "${BLKNAME}" ${CONF}/blacklist ; then
+                        return 0
+                fi
+                BLKNAME="${BLKNAME}-${PKGDATA[$i]}"
+        done
+        if grep -qx "${PKGDATA[6]/./}" ${CONF}/blacklist ; then
+		return 0
 	else
-		if [ "`echo $TDIRNAME | grep -e \"^/${FIRST}/\"`" != "" ] && [ $PKGFOUND -le 4 ]; then
-			PKGNAME="$TPKGNAME"
-			DIRNAME="$TDIRNAME"
-			TBASENAME="$TBASENAME"
-			PKGFOUND="4"
-		elif [ "`echo $TDIRNAME | grep -e \"^/${SECOND}/\"`" != "" ] && [ $PKGFOUND -le 3 ]; then
-			PKGNAME="$TPKGNAME"
-			DIRNAME="$TDIRNAME"
-			TBASENAME="$TBASENAME"
-			PKGFOUND="3"
-		elif [ "`echo $TDIRNAME | grep -e \"^/${THIRD}/\"`" != "" ] && [ $PKGFOUND -le 2 ]; then
-			PKGNAME="$TPKGNAME"
-			DIRNAME="$TDIRNAME"
-			TBASENAME="$TBASENAME"
-			PKGFOUND="2"
-		elif [ "`echo $TDIRNAME | grep -e \"^/${FOURTH}/\"`" != "" ] && [ $PKGFOUND -le 1 ]; then
-			PKGNAME="$TPKGNAME"
-			DIRNAME="$TDIRNAME"
-			TBASENAME="$TBASENAME"
-			PKGFOUND="1"
-		fi
+        	return 1
 	fi
 }
 
+function givepriority {
+        local DIR
+        local ARGUMENT=$1
+	local PKGDATA
+
+	unset NAME
+        unset FULLNAME
+	unset PKGDATA
+	
+        for DIR in $FIRST $SECOND $THIRD $FOURTH $FIFTH ; do
+		[ "$PKGDATA" ] && break
+                PKGDATA=( `grep "^${DIR} ${ARGUMENT} " ${WORKDIR}/pkglist` )
+                if [ "$PKGDATA" ]; then
+                        checkblacklist
+                        if [ "$?" = "1" ]; then
+				NAME=${PKGDATA[1]}
+                                FULLNAME=`echo "${PKGDATA[5]}"`
+			else
+				unset PKGDATA
+				unset FULLNAME
+				unset NAME
+                        fi
+                fi
+        done
+}
+
+
 # Function to make install/reinstall/upgrade lists
 #
 function makelist() {
-	ls -1 /var/log/packages > $TMPDIR/tmplist
+	local ARGUMENT
+	local i
+	local VRFY
+
+	INPUTLIST=$@
+
+	ls -1 /var/log/packages/* | awk -f /usr/libexec/slackpkg/pkglist.awk > ${TMPDIR}/tmplist
+
 	case "$CMD" in
-		install|reinstall|upgrade|download|blacklist)
-			echo -n "Looking for $(echo $1 | tr -d '\\') in package list. Please, wait... "
-			for i in `egrep -- /${1} ${CONF}/pkglist | \
-				awk -F/ '{ print $NF }'`; do
-				unset TEST
-				BASENAME=`cutpkg $i`
-
-				# That lines above choose de package with the 
-				# higher priority
-				#
-				TEST=`grep -e "^\./[a-z]*\(\|/.*\)/${BASENAME}-[^-]\+-\(noarch\|${ARCH}\)" ${CONF}/pkglist` 
-				PKGFOUND=0
-				if [ -n "$TEST" ]; then
-					for PKGTEST in $TEST ; do
-						check_blacklist_priority
-					done
-				fi
-
-				if [ "$PKGFOUND" != "0" ]; then
-					PKGFOUND=`grep -e "^${BASENAME}-[^-]\+-\(noarch\|${ARCH}\)" $TMPDIR/tmplist`.tgz
-					# Based in PKGFOUND info, put the package in
-					# the right list.
-					#
-					if [ "$PKGFOUND" = "$PKGNAME" ]; then
-						REINSTALLPKG="$REINSTALLPKG $PKGNAME"
-					elif [ "$PKGFOUND" = ".tgz" ]; then
-						INSTALLPKG="$INSTALLPKG $PKGNAME"
-					else
-						UPGRADEPKG="$UPGRADEPKG $PKGNAME"
-					fi
-				fi 
-			done
-			REINSTALLPKG=`echo -e $REINSTALLPKG | \
-							tr \  "\n" |sort|uniq`
-			INSTALLPKG=`echo -e $INSTALLPKG | tr \  "\n" |sort|uniq`
-			UPGRADEPKG=`echo -e $UPGRADEPKG | tr \  "\n" |sort|uniq`
-			;;
-		remove)
-			echo -n "Looking for packages to remove. Please, wait... "
-			REMOVEPKG=`ls -1 /var/log/packages | \
-                            egrep -i -- "^${PATTERN}.*-[^-]+-(${ARCH}|noarch)-"`
-		;;
 		clean-system)
 			echo -n "Looking for packages to remove. Please, wait... "
-	                for i in `cat $TMPDIR/tmplist`; do
-				PKGNAME="`cutpkg $i | \
-					sed -e 's/\+/\\\+/g' -e 's/\./\\\./g'`"
-				if [ `egrep -c "${PKGNAME}-[^-]+-(${ARCH}|noarch)-" $CONF/pkglist` = "0" ] && [ `grep -cx "${PKGNAME}" $CONF/blacklist` = 0 ]; then
-					REMOVEPKG="$REMOVEPKG $i"
+		;;
+		upgrade-all)
+			echo -n "Looking for packages to upgrade. Please, wait... "
+		;;
+		install-new)
+			echo -n "Looking for NEW packages to install. Please, wait... "
+		;;
+		*)
+			echo -n "Looking for $(echo $INPUTLIST | tr -d '\\') in package list. Please, wait... "
+		;;
+	esac
+
+	case "$CMD" in
+		download)
+			for ARGUMENT in `echo $INPUTLIST`; do
+				for i in `grep -w -- "${ARGUMENT}" ${WORKDIR}/pkglist | cut -f2 -d\  | sort -u`; do
+					LIST="$LIST `grep " ${i} " ${WORKDIR}/pkglist | cut -f6 -d \ `"
+				done
+				LIST="`echo -e $LIST | sort -u`"
+			done
+		;;
+		blacklist)
+			for ARGUMENT in `echo $INPUTLIST`; do
+				for i in `cat ${WORKDIR}/pkglist ${TMPDIR}/tmplist | \
+						grep -w -- "${ARGUMENT}" | cut -f2 -d\  | sort -u`; do
+					grep -qx "${i}" ${CONF}/blacklist || LIST="$LIST $i"
+				done
+			done
+		;;
+		install|upgrade|reinstall)
+			for ARGUMENT in `echo $INPUTLIST`; do
+				for i in `grep -w -- "${ARGUMENT}" ${WORKDIR}/pkglist | cut -f2 -d\  | sort -u`; do
+					givepriority $i
+					[ ! "$FULLNAME" ] && continue
+
+					case $CMD in
+						'upgrade')
+							VRFY=`cut -f6 -d\  ${TMPDIR}/tmplist | \
+							      grep -x "${NAME}-[^-]\+-\(noarch\|${ARCH}\)-[^-]\+"`
+							[ "${FULLNAME}" != "${VRFY}" ]  && \
+										[ "${VRFY}" ] && \
+								LIST="$LIST ${FULLNAME}"
+						;;
+						'install')
+							grep -q " ${NAME} " ${TMPDIR}/tmplist || \
+								LIST="$LIST ${FULLNAME}"
+						;;
+						'reinstall')
+							grep -q " ${FULLNAME} " ${TMPDIR}/tmplist && \
+								LIST="$LIST ${FULLNAME}"
+						;;
+					esac
+				done
+			done
+		;;
+		remove)
+			for ARGUMENT in `echo $INPUTLIST`; do
+				for i in `cat ${WORKDIR}/pkglist ${TMPDIR}/tmplist | \
+					  	grep -w -- "${ARGUMENT}" | cut -f6 -d\  | sort -u`; do
+					PKGDATA=( `grep -w -- "$i" ${TMPDIR}/tmplist` )
+					[ ! "$PKGDATA" ] && continue
+					checkblacklist
+					LIST="$LIST ${PKGDATA[5]}" 
+					unset PKGDATA
+				done
+			done
+		;;
+		clean-system)
+	                for i in `cut -f6 -d\  ${TMPDIR}/tmplist`; do
+				NAME=`cutpkg $i`
+				if [ `cut -f2 -d\  ${WORKDIR}/pkglist |grep -cx "${NAME}"` = "0" ] &&
+					[ `grep -cx "${NAME}" $CONF/blacklist` = 0 ]; then
+					LIST="$LIST $i"
                         	fi
                 	done
 		;;
 		upgrade-all)
-			echo -n "Looking for packages to upgrade. Please, wait... "
-			for i in `cat $TMPDIR/tmplist`; do
-				unset TEST
-				PKGNAME=`cutpkg $i`
-				TEST=`grep -e "^\./[a-z]*\(\|/.*\)/${PKGNAME}-[^-]\+-\(noarch\|${ARCH}\)" ${CONF}/pkglist` 
-				PKGFOUND=0
-				if [ -n "$TEST" ]; then
-					for PKGTEST in $TEST ; do
-						check_blacklist_priority
-					done
-				fi
-				if [ "${PKGNAME/.tgz/}" != "$i" ] && [ -n "$TEST" ] && [ "$PKGFOUND" != "0" ]; then 
-					UPGRADEPKG="$UPGRADEPKG $PKGNAME"
-				fi
+			for i in `cut -f2 -d\  ${TMPDIR}/tmplist`; do
+
+				givepriority ${i}
+				[ ! "$FULLNAME" ] && continue
+
+				VRFY=`cut -f6 -d\  ${TMPDIR}/tmplist | grep -x "${NAME}-[^-]\+-\(noarch\|${ARCH}\)-[^-]\+"`
+				[ "${FULLNAME}" != "${VRFY}" ]  && \
+							[ "${VRFY}" ] && \
+					LIST="$LIST ${FULLNAME}"
 			done
-			UPGRADEPKG=`echo -e $UPGRADEPKG | tr \  "\n"|sort|uniq`
 		;;
 		install-new)
-			echo -n "Looking for NEW packages to install. Please, wait... "
-			for i in `grep -v "testing" $CONF/ChangeLog.txt | \
-				  awk -F: '/[a-z].*\/([a-z_]+)-.*:.*(Added|Split|Renamed).*/ \
-				  { print $1 }'` \
-				  a/acl a/attr ap/dmapi ap/xfsdump l/mm ; do    
-				unset INSNAME
-				PKGNAME=`echo $i | awk -F/ '{ print $NF }'`
-				DIRNAME=`dirname $i`
-				case $DIRNAME in
-					a|ap|d|e|f|k|kde|kdei|l|n|t|tcl|x|xap|y)
-						DIRNAME="/slackware/${DIRNAME}"
-						;;
-					*)
-						DIRNAME="/${DIRNAME}"
-						;;
-				esac
-				BASENAME=`cutpkg $PKGNAME`
-
-				INSNAME=`ls -1 /var/log/packages | grep -e "^${BASENAME}-[^-]\+-\(${ARCH}\|noarch\)-"`
-				if [ "${INSNAME}" = "" ] && \
-				   [ `grep -cx "\(${BASENAME}\|${DIRNAME}\)" $CONF/blacklist` = 0 ]; then
-					NAMEPKG=`basename $(grep -e "${DIRNAME}/${BASENAME}-[^-]\+-\(${ARCH}\|noarch\)-" $CONF/pkglist) 2>/dev/null`
-					if [ "$NAMEPKG" != "" ] && \
-					   [ `echo $INSTALLPKG | grep -c "$NAMEPKG"` = 0 ]; then
-						INSTALLPKG="$INSTALLPKG $NAMEPKG"
-					fi
-				fi
+			for i in `awk -f /usr/libexec/slackpkg/install-new.awk ${WORKDIR}/ChangeLog.txt |\
+				  sort -u ` dialog aaa_terminfo fontconfig ; do
+	
+				givepriority $i
+				[ ! "$FULLNAME" ] && continue
+				
+				grep -q " ${NAME} " ${TMPDIR}/tmplist || \
+					LIST="$LIST ${FULLNAME}"
 			done
 		;;
 	esac
+	LIST=`echo -e $LIST | tr \  "\n"`
 	echo -e "DONE\n"
 }
 
 # Function to count total of packages
 #
 function countpkg() {
-	COUNTPKG=`echo -e "$1" | wc -w`
+	local COUNTPKG=`echo -e "$1" | wc -w`
+
 	if [ "$COUNTPKG" != "0" ]; then
 		echo -e "Total of package(s): $COUNTPKG\n"
 	fi
@@ -401,6 +437,9 @@ function countpkg() {
 # Return accepted list in $SHOWLIST
 #
 function showlist() {
+	local ANSWER
+	local i
+
 	for i in $1; do echo $i; done | more 
 	echo
 	countpkg "$1"
@@ -419,33 +458,37 @@ function getfile() {
                 cp ${SOURCE}$1 $2 2>/dev/null
         else
                 echo -e "\t\t\tDownloading $1..."
-                wget $WGETFLAGS ${SOURCE}$1 -O $2
+                wget ${WGETFLAGS} ${SOURCE}$1 -O $2
         fi
 }                                                       
 
 # Function to download the correct package and many "checks"
 #
 function getpkg() {
-	ISOK="1"
-	NAMEPKG=`grep -m 1 -- "/$1$" ${CONF}/pkglist`
-	
+	local ISOK="1"
+	local PKGNAME=( `grep -w -m 1 -- "$1" ${WORKDIR}/pkglist` )
+	local FULLPATH
+
+	NAMEPKG=${PKGNAME[5]}
+	FULLPATH=${PKGNAME[6]}
+
 	if [ "`ls ${TEMP}/$1 2>/dev/null`" = "" ]; then
 		echo -e "\nPackage: $1"
 		# Check if the mirror are local, if is local, copy files 
-		# to $TEMP else, download packages from remote host and 
-		# put then in $TEMP
+		# to TEMP else, download packages from remote host and 
+		# put then in TEMP
 		#
-		if [ "$LOCAL" = "1" ]; then 
+		if [ "${LOCAL}" = "1" ]; then 
                 	echo -e "\tCopying $1..."
-			cp ${SOURCE}${NAMEPKG} $TEMP
-			if [ "$NOGPG" != "1" ]; then
-				cp ${SOURCE}${NAMEPKG}.asc $TEMP
+			cp ${SOURCE}${FULLPATH}/${NAMEPKG} ${TEMP}
+			if [ "$CHECKGPG" = "on" ]; then
+				cp ${SOURCE}${FULLPATH}/${NAMEPKG}.asc ${TEMP}
 			fi
 		else
                 	echo -e "\tDownloading $1..."
-			wget $WGETFLAGS -P $TEMP -nd ${SOURCE}${NAMEPKG}
-			if [ "$NOGPG" != "1" ]; then
-				wget $WGETFLAGS -P $TEMP -nd ${SOURCE}${NAMEPKG}.asc
+			wget ${WGETFLAGS} -P ${TEMP} -nd ${SOURCE}${FULLPATH}/${NAMEPKG}
+			if [ "$CHECKGPG" = "on" ]; then
+				wget ${WGETFLAGS} -P ${TEMP} -nd ${SOURCE}${FULLPATH}/${NAMEPKG}.asc
 			fi
 		fi
 	else
@@ -455,7 +498,7 @@ function getpkg() {
 	# If MD5SUM checks are enabled in slackpkg.conf, check the
 	# packages md5sum to detect if they are corrupt or not
 	#
-	if [ "$CHECKPKG" = "1" ]; then
+	if [ "$CHECKPKG" = "on" ]; then
 		ISOK=`checkpkg $1`
 		[ "$ISOK" = "0" ] && ERROR="md5sum"
 	fi
@@ -463,7 +506,7 @@ function getpkg() {
 	# Check the package against its .asc. If you don't like this
 	# disable GPG checking in /etc/slackpkg/slackpkg.conf
 	#
-	if [ "$NOGPG" != "1" ]; then
+	if [ "$CHECKGPG" = "on" ]; then
 		ISOK=`checkgpg $1`
 		[ "$ISOK" = "0" ] && ERROR="gpg"
 	fi
@@ -489,7 +532,7 @@ function getpkg() {
 	# If DELALL is checked, all downloaded files will be erased
 	# after installed/upgraded/reinstalled
 	#
-	if [ "$DELALL" = "1" ]; then
+	if [ "$DELALL" = "on" ]; then
 		rm $TEMP/$1 $TEMP/${1}.asc 2>/dev/null
 	fi		
 }
@@ -498,8 +541,8 @@ function getpkg() {
 #
 function updatefilelists()
 {
-	if ! [ -e $CONF/ChangeLog.txt ]; then
-		touch $CONF/ChangeLog.txt
+	if ! [ -e ${WORKDIR}/ChangeLog.txt ]; then
+		touch ${WORKDIR}/ChangeLog.txt
 	fi
 
 	echo -e "\tDownloading..."
@@ -508,7 +551,7 @@ function updatefilelists()
 	# or different from our already existent ChangeLog.txt 
 	#
 	getfile ChangeLog.txt $TMPDIR/ChangeLog.txt
-	if diff --brief $CONF/ChangeLog.txt $TMPDIR/ChangeLog.txt ; then
+	if diff --brief ${WORKDIR}/ChangeLog.txt $TMPDIR/ChangeLog.txt ; then
 		echo -e "\
 \n\t\tNo changes in ChangeLog.txt between your last update and now.\n\
 \t\tDo you really want to download all other files (y/N)? \c"
@@ -518,17 +561,16 @@ function updatefilelists()
 		fi
 	fi
 	echo
-	cp $TMPDIR/ChangeLog.txt $CONF/ChangeLog.txt
+	cp $TMPDIR/ChangeLog.txt ${WORKDIR}/ChangeLog.txt
 
 	#
 	# Download MANIFEST, FILELIST.TXT and CHECKSUMS.md5
 	#
 
-	# That will be download MANIFEST.bz2 files in /slackware
-	# /extra and /pasture. 
+	# That will be download MANIFEST.bz2 files
 	#
 	echo -e "\t\tList of all files"
-	for i in patches slackware extra pasture; do
+	for i in $FIRST $SECOND $THIRD $FOURTH $FIFTH ; do 
 		getfile ${i}/MANIFEST.bz2 $TMPDIR/${i}-MANIFEST.bz2 && \
 			DIRS="$DIRS $i"
 	done
@@ -536,9 +578,9 @@ function updatefilelists()
 	echo -e "\t\tPackage List"
 	getfile FILELIST.TXT $TMPDIR/FILELIST.TXT
 
-	if [ "$CHECKPKG" = "1" ]; then
+	if [ "$CHECKPKG" = "on" ]; then
 		echo -e "\t\tChecksums"
-		getfile CHECKSUMS.md5 ${CONF}/CHECKSUMS.md5
+		getfile CHECKSUMS.md5 ${WORKDIR}/CHECKSUMS.md5
 	fi
 		
 	# Download all PACKAGES.TXT files
@@ -552,7 +594,7 @@ function updatefilelists()
 	#
 	echo -e "\tFormatting lists to slackpkg style..."
 	echo -e "\t\tPackage List"
-	awk '/tgz/ && !/source\// && !/testing\// && !/.asc/ {print $NF}' $TMPDIR/FILELIST.TXT | tr -d "\r" > $CONF/pkglist
+	grep "\.tgz" $TMPDIR/FILELIST.TXT| awk -f /usr/libexec/slackpkg/pkglist.awk > ${WORKDIR}/pkglist 
 		
 	# Format MANIFEST
 	#
@@ -561,13 +603,13 @@ function updatefilelists()
 	#
 	MANFILES=""
 	for i in $DIRS; do
-		bunzip2 -c $TMPDIR/${i}-MANIFEST.bz2 | awk -f $CONF/filelist.awk | \
-			gzip > $CONF/${i}-filelist.gz
+		bunzip2 -c $TMPDIR/${i}-MANIFEST.bz2 | awk -f /usr/libexec/slackpkg/filelist.awk | \
+			gzip > ${WORKDIR}/${i}-filelist.gz
 	done
 
-	if [ -r ${CONF}/filelist.gz ]; then
-		rm $CONF/filelist.gz
-		ln -s $CONF/slackware-filelist.gz $CONF/filelist.gz
+	if [ -r ${WORKDIR}/filelist.gz ]; then
+		rm ${WORKDIR}/filelist.gz
+		ln -s ${WORKDIR}/${MAIN}-filelist.gz ${WORKDIR}/filelist.gz
 	fi
 
 	# Concatenate PACKAGE.TXT files
@@ -576,17 +618,23 @@ function updatefilelists()
 	for i in $DIRS; do
 		cat $TMPDIR/${i}-PACKAGES.TXT >> $TMPDIR/PACKAGES.TXT
 	done
-	cp $TMPDIR/PACKAGES.TXT $CONF/PACKAGES.TXT
+	cp $TMPDIR/PACKAGES.TXT ${WORKDIR}/PACKAGES.TXT
 }
 
 function sanity_check() {
+	local REVNAME
+	local i
+	local FILES
+	local DOUBLEFILES
+	local ANSWER
+
 	for i in `ls -1 /var/log/packages | \
 		egrep -- "^.*-(${ARCH}|noarch)-[^-]+-upgraded"`; do
 		REVNAME=`echo ${i} | awk -F'-upgraded' '{ print $1 }'`
 		mv /var/log/packages/${i} /var/log/packages/${REVNAME}
 		mv /var/log/scripts/${i} /var/log/scripts/${REVNAME}
 	done 
-	for i in `ls -1 /var/log/packages | egrep '\-[^\.]+$'`; do
+	for i in `ls -1 /var/log/packages | egrep "^.*-(${ARCH}|noarch)-[^-]+$"`; do
 		cutpkg $i
 	done | sort > $TMPDIR/list1
 	cat $TMPDIR/list1 | uniq > $TMPDIR/list2
@@ -637,21 +685,15 @@ Ok! slackpkg won't do anything now. But, please, do something to fix it.\n"
 }	
 
 function blacklist_pkg() {
-	BLACKLISTFILE="$TMPDIR/blacklist"
-	echo $SHOWLIST | tr ' ' "\n" > ${BLACKLISTFILE}
-					   
-	grep "^#" ${CONF}/blacklist > ${BLACKLISTFILE}.new
-	grep -v "^#" ${CONF}/blacklist >> ${BLACKLISTFILE}
-
-	cat ${BLACKLISTFILE} | grep -v "^$" | sort | \
-		uniq >> ${BLACKLISTFILE}.new
-	mv ${BLACKLISTFILE}.new ${CONF}/blacklist
+	echo $SHOWLIST | tr ' ' "\n" >> ${CONF}/blacklist
 
 	echo -e "\nPackages added to your blacklist.\n\
 If you want to remove those packages, edit ${CONF}/blacklist.\n"
 }
 
 function remove_pkg() {
+	local i
+
 	for i in $SHOWLIST; do
 		echo -e "\nPackage: $i"
 		echo -e "\tRemoving... "
@@ -660,9 +702,11 @@ function remove_pkg() {
 }
 
 function upgrade_pkg() {
-	if [ "$DOWNLOAD_ALL" = "1" ]; then
+	local i
+
+	if [ "$DOWNLOAD_ALL" = "on" ]; then
 		OLDDEL="$DELALL"
-		DELALL="0"
+		DELALL="off"
 		for i in $SHOWLIST; do
 			getpkg $i true
 		done
@@ -674,9 +718,11 @@ function upgrade_pkg() {
 }
 
 function install_pkg() {
-	if [ "$DOWNLOAD_ALL" = "1" ]; then
+	local i
+
+	if [ "$DOWNLOAD_ALL" = "on" ]; then
 		OLDDEL="$DELALL"
-		DELALL="0"
+		DELALL="off"
 		for i in $SHOWLIST; do
 			getpkg $i true
 		done
